use axum::{
    http::header::SET_COOKIE,
    response::{AppendHeaders, IntoResponse, Redirect},
    routing::post,
    Extension, Form, Router,
};

use lettre::message::Mailbox;
use lettre::transport::smtp::AsyncSmtpTransport;
use lettre::{AsyncTransport, Message, Tokio1Executor};
use serde::Deserialize;
use std::error::Error;
use std::net::SocketAddr;
use std::sync::Arc;

#[derive(Deserialize)]
#[serde(rename_all = "kebab-case")]
struct FormData {
    name: Option<String>,
    email: String,
    company_name: Option<String>,
    service: String,
    help: String,
    hear_about: Option<String>,
    other_source: Option<String>,
}

struct State {
    username: String,
    password: String,
}

#[tokio::main]
async fn main() -> Result<(), Box<dyn std::error::Error>> {
    let username = std::env::var("WEBMAIL_USERNAME")?;
    let password = std::env::var("WEBMAIL_PASSWORD")?;

    let state = State { username, password };

    let app = Router::new()
        .route("/submit-email", post(handle_form))
        .layer(Extension(Arc::new(state)));

    let addr = SocketAddr::from(([0, 0, 0, 0], 4200));
    println!("Listening on {}", addr);

    let listener = tokio::net::TcpListener::bind("0.0.0.0:4200").await?;
    axum::serve(listener, app).await?;

    Ok(())
}

async fn handle_form(
    state: Extension<Arc<State>>,
    Form(form_data): Form<FormData>,
) -> impl IntoResponse {
    match send_email(form_data, state.as_ref()).await {
        Ok(_) => (
            AppendHeaders([(SET_COOKIE, "is=ok")]),
            Redirect::to("/contact.html"),
        ),
        Err(e) => {
            eprintln!("Encountered error sending mail: {e:?}");
            (
                AppendHeaders([(SET_COOKIE, "is=err")]),
                Redirect::to("/contact.html"),
            )
        }
    }
}

async fn send_email(form_data: FormData, state: &State) -> Result<(), Box<dyn Error>> {
    let username = &state.username;
    let password = &state.password;

    let email = form_data.email.parse::<Mailbox>()?;

    let content = [
        (
            "Name?",
            form_data.name.unwrap_or("Not provided".to_string()),
        ),
        ("Email", form_data.email),
        (
            "Company Name?",
            form_data.company_name.unwrap_or("Not provided".to_string()),
        ),
        ("Service", form_data.service),
        ("How can we help?", form_data.help),
        ("How did you hear about us?", form_data.hear_about),
        (
            "Other",
            form_data
                .other_source
                .unwrap_or("Not provided/relevant".to_string()),
        ),
    ];
    let body = content.map(|(a, b)| format!("{a}: {b}")).join("\n");

    println!("Body: {body}");

    let email = Message::builder()
        .from(username.parse()?)
        .reply_to(email)
        .to("ryan@distrust.co".parse()?)
        .cc("anton@distrust.co".parse()?)
        .subject("New Website Form Inquiry")
        .body(body)?;

    let credentials = lettre::transport::smtp::authentication::Credentials::new(
        username.clone(),
        password.clone(),
    );

    let mailer = AsyncSmtpTransport::<Tokio1Executor>::relay("smtp.migadu.com")?
        .credentials(credentials)
        .build();

    let response = mailer.send(email).await?;
    dbg!(&response);

    Ok(())
}
